import { useRef } from "react";
import {
  IgrColumn,
  IgrGrid,
  IgrGridModule,
  IgrSortingExpressionEventArgs,
  IgrSortingStrategy,
  SortingDirection,
} from "igniteui-react-grids";

import "igniteui-react-grids/grids/combined";
import "igniteui-react-grids/grids/themes/light/bootstrap.css";
import { DATA } from "./assets/data";

IgrGridModule.register();

function App() {
  const gridRef = useRef<IgrGrid>(null);

  function handleSortingExpressionsChange(
    s: IgrGrid,
    e: IgrSortingExpressionEventArgs
  ) {
    if (e.detail.length) {
      const characteristicsExpression = e.detail.find(
        (expression) => expression.fieldName === "Characteristics"
      );

      if (characteristicsExpression) {
        characteristicsExpression.strategy = customSorting;
      }
    }
  }

  const customSorting: IgrSortingStrategy = {
    sort(
      data: any[],
      fieldName: string,
      dir: SortingDirection,
      ignoreCase: boolean
    ): any[] {
      const reverse = dir === SortingDirection.Desc ? -1 : 1;
      const sorted = data.sort((a, b) => {
        a = a.Characteristics.ProductName;
        b = b.Characteristics.ProductName;

        if (ignoreCase) {
          a = a.toLowerCase();
          b = b.toLowerCase();
        }

        return reverse * compareValues(a, b);
      });

      return sorted;
    },
  };

  function compareValues(a: any, b: any): number {
    const an = a === null || a === undefined;
    const bn = b === null || b === undefined;
    if (an) {
      if (bn) {
        return 0;
      }
      return -1;
    } else if (bn) {
      return 1;
    }
    return a > b ? 1 : a < b ? -1 : 0;
  }

  // Using formatter to display the complex data,
  // otherwise it is displayed as [Object object].
  function formatter(value: any, rowData: any): any {
    return `${rowData.Characteristics.ProductName} - *${rowData.Characteristics.Team}*`;
  }

  return (
    <>
      <IgrGrid
        id="grid1"
        ref={gridRef}
        data={DATA}
        primaryKey="ID"
        height="480px"
        width="700px"
        sortingExpressionsChange={handleSortingExpressionsChange}
      >
        <IgrColumn field="ProductID" header="Product ID"></IgrColumn>
        <IgrColumn
          field="Characteristics"
          header="Characteristics"
          sortable={true}
          formatter={formatter}
        ></IgrColumn>
      </IgrGrid>
    </>
  );
}

export default App;
